#ifndef MsgStructure_HPP
#define MsgStructure_HPP

#include <string>
#ifndef Q_MOC_RUN
#include <boost/regex.hpp>
#endif

using namespace std;

// **** Symbols ****
static const string SPACE		= "[[:space:]]";
static const string LSCOPE		= "[{]";
static const string RSCOPE		= "[}]";
static const string LBRAC		= "[[.left-square-bracket.]]";
static const string RBRAC		= "[[.right-square-bracket.]]";
static const string COLON		= "[[.colon.]]";
static const string COMMA		= "[[.comma.]]";
static const string QUOTE		= "[[.quotation-mark.]]";
static const string WORD		= "([[:word:]]+)";
static const string STRING		= QUOTE + WORD + QUOTE;
static const string SLIST		= LBRAC + "(((" + STRING + ")|(" + STRING + COMMA + SPACE + "?" + "))*)" + RBRAC;
static const boost::regex wordRegex(WORD);

// **** HATP message ****
static const string HATP_REP		= QUOTE + "HATP-REP" + QUOTE + COLON + SPACE;
static const string RESULT		= QUOTE + "RESULT" + QUOTE + COLON + SPACE;
static const string TASK_NAME		= QUOTE + "TASK-NAME" + QUOTE + COLON + SPACE;
static const string ENTITY_START	= QUOTE + "ENTITY" + QUOTE + COLON + SPACE;
static const string PARAMETERS		= QUOTE + "PARAMETERS" + QUOTE + COLON + SPACE;
static const string TYPE_DELIMITER	= QUOTE + "TYPE" + QUOTE + COLON + SPACE;

// **** Method answer ****
static const string METHOD_NAME		= STRING + COMMA + SPACE;
static const string METHOD		= LSCOPE + TASK_NAME + METHOD_NAME + PARAMETERS + SLIST + RSCOPE;
static const string METHOD_SET		= LBRAC + "(((" + METHOD + ")|(" + METHOD + COMMA + SPACE + "?" + "))*)" + RBRAC;
static const string METHOD_ANSWER	= LSCOPE + HATP_REP + LSCOPE + RESULT + METHOD_SET + RSCOPE + RSCOPE;
static const boost::regex methodRegex(METHOD);
static const boost::regex methodAnswerRegex(METHOD_ANSWER);

// **** Actions answer ****
static const string ACTION_NAME		= STRING + COMMA + SPACE;
static const string ACTION		= LSCOPE + TASK_NAME + ACTION_NAME + PARAMETERS + SLIST + RSCOPE;
static const string ACTION_SET		= LBRAC + "(((" + ACTION + ")|(" + ACTION + COMMA + SPACE + "?" + "))*)" + RBRAC;
static const string ACTION_ANSWER	= LSCOPE + HATP_REP + LSCOPE + RESULT + ACTION_SET + RSCOPE + RSCOPE;
static const boost::regex actionRegex(ACTION);
static const boost::regex actionAnswerRegex(ACTION_ANSWER);

// **** Entity answer ****
static const string ENTITY_NAME		= STRING + COMMA + SPACE;
static const string ENTITY		= LSCOPE + ENTITY_START + ENTITY_NAME + TYPE_DELIMITER + STRING + RSCOPE;
static const string ENTITY_SET		= LBRAC + "(((" + ENTITY + ")|(" + ENTITY + COMMA + SPACE + "?" + "))*)" + RBRAC;
static const string ENTITY_ANSWER	= LSCOPE + HATP_REP + LSCOPE + RESULT + ENTITY_SET + RSCOPE + RSCOPE;
static const boost::regex entityRegex(ENTITY);
static const boost::regex entityAnswerRegex(ENTITY_ANSWER);

// **** Command message ****
static const string COMMAND		= QUOTE + "COMMAND" + QUOTE + COLON;
static const string EXIT		= QUOTE + "EXIT" + QUOTE;
static const string CONNECT		= QUOTE + "CONNECT" + QUOTE;
static const string EXIT_COMMAND	= LSCOPE + COMMAND + EXIT + RSCOPE;
static const string CONNECT_COMMAND	= LSCOPE + COMMAND + CONNECT + RSCOPE;
static const boost::regex exitCommandRegex(EXIT_COMMAND);
static const boost::regex connectCommandRegex(CONNECT_COMMAND);

#endif //MsgStructure_HPP
